/*	PROJET P3 - IMA4 - Florian GIOVANNANGELI 	*/
/*	Programme émetteur				*/

#include <SPI.h>
#include <RH_RF95.h>
#include <RTCZero.h>
#include <Wire.h>
#include "SparkFunHTU21D.h"
#include "FXLS8471Q-P.h"


/*_____________ For Feather M0 _____________*/
#define RFM95_CS 8
#define RFM95_RST 4
#define RFM95_INT 3

// Choose frequency, must match RX's freq
#define RF95_FREQ 434.0
// Singleton instance of the radio driver
RH_RF95 rf95(RFM95_CS, RFM95_INT);

/*________ For RTC : PLEASE SET TIME ________*/
RTCZero rtc;
int8_t hours = 8 ;    // Set hour
int8_t minutes = 00 ;  // Set minute
int8_t seconds = 00 ;  // Set second
int8_t day = 1 ;     // Set day
int8_t month = 1 ;   // Set month
int8_t year = 17 ;    // Set year (AA)

bool alarmTriggered = true;
int8_t alarmHour = hours ;
int8_t alarmMinute = minutes ;
int8_t alarmDay = day ;
int8_t alarmMonth = month ;
int8_t alarmYear = year ;

/*____________ For the sensors ____________*/
// Temperature/Humidity sensor HTU21D
HTU21D myHumidity;
FXLS8471Q myAccel;


/*______________ Miscellaneous ______________*/
int16_t packetnum = 0;  // packet counter, we increment per xmission

int nbHour = 0 ;
int nbDay = 0 ;
bool reply ;




/*-------------------------------------------------- 
|                       SETUP                      |
---------------------------------------------------*/
void setup()
{
  /* STARTING */
  pinMode(RFM95_RST, OUTPUT);
  digitalWrite(RFM95_RST, HIGH);

  //while (!Serial);		// TO REMOVE for the transmitter
  Serial.begin(9600);
  delay(100);

  // manual reset
  digitalWrite(RFM95_RST, LOW);
  delay(10);
  digitalWrite(RFM95_RST, HIGH);
  delay(10);

  
  /* INITIALIZING RADIO */
  while (!rf95.init()) {
    Serial.println("LoRa radio init failed");
    while (1);
  }
  Serial.println("LoRa radio init OK!");
  
  // Defaults after init are 434.0MHz, modulation GFSK_Rb250Fd250, +13dbM
  if (!rf95.setFrequency(RF95_FREQ)) {
    Serial.println("setFrequency failed");
    while (1);
  }
    Serial.print("Set Freq to: "); Serial.println(RF95_FREQ);
    // Defaults after init are 434.0MHz, 13dBm, Bw = 125 kHz, Cr = 4/5, Sf = 128chips/symbol, CRC on
  
  // The default transmitter power is 13dBm, using PA_BOOST.
  // If you are using RFM95/96/97/98 modules which uses the PA_BOOST transmitter pin, then 
  // you can set transmitter powers from 5 to 23 dBm:
  rf95.setTxPower(23, false);


  /* INITIALIZING HTU21D SENSOR */
  myHumidity.begin();
  /* INITIALIZING FXLS8471Q SENSOR */
  myAccel.begin();
 
  /* INITIALIZING RTC */
  rtc.begin(); // To use the RTC, first begin it
  rtc.setTime(hours, minutes, seconds); // Then set the time
  rtc.setDate(day, month, year); // And the date
  Serial.println("RTC Started!");
  /* ALARM INITIALIZING */
  rtc.setAlarmTime(alarmHour, alarmMinute, 0);
  rtc.setAlarmDay(alarmDay);
  rtc.setAlarmMonth(alarmMonth);

  rtc.enableAlarm(rtc.MATCH_MMDDHHMMSS); // Alarm when month, day, hours, minute, & second match
  // When the alarm triggers, wakeCallback will be called:
  rtc.attachInterrupt(wakeCallback);

}



/*-------------------------------------------------- 
|                        LOOP                      |
---------------------------------------------------*/
void loop()
{
  if (alarmTriggered) // If the alarm has been triggered
  {
    /* RESET AND NEW LOOP */
    delay(10);
    alarmTriggered = false;
    rtc.disableAlarm();
    
    do{             // Do while no reply message
      delay(1000);  // Wait 1 second between transmits, could also 'sleep' here

      /* READING AND SENDING DATA */
      float Temp = myHumidity.readTemperature() ;
      unsigned char* pT = (unsigned char*)&Temp ;
      
      float Humd = myHumidity.readHumidity() ;
      unsigned char* pH = (unsigned char*)&Humd ;

      float X;
      unsigned char* pX = (unsigned char*)&X ;
      float Y;
      unsigned char* pY = (unsigned char*)&Y ;
      float Z;
      unsigned char* pZ = (unsigned char*)&Z ;
      myAccel.ReadAccel(&X, &Y, &Z);

      uint8_t RadioPacket[20];    // Packet to send
      
           
      memcpy(RadioPacket,pT,sizeof(Temp));                    // Copy 4 bytes of Temp
      memcpy(RadioPacket+sizeof(Temp),pH,sizeof(Humd));       // Copy 4 bytes of Humd
      memcpy(RadioPacket+sizeof(Temp)+sizeof(Humd),pX,sizeof(X));     // Copy 4 bytes of X
      memcpy(RadioPacket+sizeof(Temp)+sizeof(Humd)+sizeof(X),pY,sizeof(Y));   // Copy 4 bytes of Y
      memcpy(RadioPacket+sizeof(Temp)+sizeof(Humd)+sizeof(X)+sizeof(Y),pZ,sizeof(Z));   // Copy 4 bytes of Z

      rf95.send((uint8_t *)&RadioPacket, sizeof(RadioPacket)); // Sending Data
      
      rf95.waitPacketSent();  // Data sent
      delay(10); 
  
      // Now wait for a reply
      uint8_t buf[RH_RF95_MAX_MESSAGE_LEN];
      uint8_t len = sizeof(buf);
      if (rf95.waitAvailableTimeout(1000))
      { 
        // Should be a reply message for us now   
        if (rf95.recv(buf, &len))
        {
          reply = true ;    
        }
        else
        {
          reply = false ;
        }
      }
      else
      {
        reply = false ;
      }
    }while(!reply); // Send data while no reply

    /* PREPARE TO SET NEW ALARM */
    int8_t alarmHour = rtc.getHours() ;
    int8_t alarmMinute = rtc.getMinutes() ;
    int8_t alarmDay = rtc.getDay() ;
    int8_t alarmMonth = rtc.getMonth() ;
    
    /* CHOICE OF CASE FOR STANDBY */
    if (nbDay >= 100){       // Step 3 (>100 days) = Data once a month
      alarmHour = 8 ;
      alarmMonth += 1 % 12 ;
    }
    if (nbDay >= 30 && nbDay < 100){  // Step 2 (>30 days) = Data once a day
      alarmDay += 1 % 30 ;
      alarmHour = 8 ;
      nbDay += 1;
    }
    else      // Step 1 (Start -> 30 days) = Data once per hour
    {
      alarmHour += 1 ;
      alarmHour = alarmHour % 24; // checks for roll over 24 hours and corrects
      nbHour +=1 ;
    }

    // Calculation of the number of days from the beginning
    if (nbHour == 24){
      nbHour = 0 ;
      nbDay += 1 ;
    }
    
    /* ALARM SETTING */
    rtc.setAlarmTime(alarmHour, alarmMinute, 0);
    rtc.setAlarmDay(alarmDay);
    rtc.setAlarmMonth(alarmMonth);

    rtc.enableAlarm(rtc.MATCH_MMDDHHMMSS); // Alarm when month, day, hours, minute, & second match
    // When the alarm triggers, wakeCallback will be called:
    rtc.attachInterrupt(wakeCallback);
    rtc.standbyMode();    // Sleep until next alarm match
  }
  reply = false ;
  delay(10);
}




/*-------------------------------------------------- 
|                WAKEUP FUNCTION                   |
---------------------------------------------------*/
void wakeCallback() {
  // This function is called when the alarm values match and the alarm is triggered.
    alarmTriggered = true; // Set the global triggered flag
}


