/*  PROJET P3 - IMA4 - Florian GIOVANNANGELI    */
/*  Programme récepteur                         */


#include <SPI.h>
#include <RH_RF95.h>
#include <RTCZero.h>

/* for Feather M0 */
#define RFM95_CS 8
#define RFM95_RST 4
#define RFM95_INT 3

// Choose frequency, must match RX's freq
#define RF95_FREQ 434.0
// Singleton instance of the radio driver
RH_RF95 rf95(RFM95_CS, RFM95_INT);
// Blinky on receipt
#define LED 13

/* for RTC : PLEASE SET TIME */
RTCZero rtc;
int8_t hours = 11 ;    // Set hour
int8_t minutes = 00 ;  // Set minute
int8_t seconds = 00 ;  // Set second
int8_t day = 19 ;     // Set day
int8_t month = 06 ;   // Set month
int8_t year = 17 ;    // Set year (AA)

int n=0 ; // Data Receipt Number


/*---------------------------- SETUP ----------------------------*/
void setup()
{
  pinMode(LED, OUTPUT);     
  pinMode(RFM95_RST, OUTPUT);
  digitalWrite(RFM95_RST, HIGH);
  
  while (!Serial);
  Serial.begin(9600);
  delay(100);
  Serial.println("Feather LoRa RX Test!");
  
  // manual reset
  digitalWrite(RFM95_RST, LOW);
  delay(10);
  digitalWrite(RFM95_RST, HIGH);
  delay(10);
  
  while (!rf95.init()) {
    Serial.println("LoRa radio init failed");
    while (1);
  }
  Serial.println("LoRa radio init OK!");
  
  // Defaults after init are 434.0MHz, modulation GFSK_Rb250Fd250, +13dbM
  if (!rf95.setFrequency(RF95_FREQ)) {
        Serial.println("setFrequency failed");
    while (1);
  }
  Serial.print("Set Freq to: "); Serial.println(RF95_FREQ);
  // Defaults after init are 434.0MHz, 13dBm, Bw = 125 kHz, Cr = 4/5, Sf = 128chips/symbol, CRC on
  
  // The default transmitter power is 13dBm, using PA_BOOST.
  // If you are using RFM95/96/97/98 modules which uses the PA_BOOST transmitter pin, then 
  // you can set transmitter powers from 5 to 23 dBm:
  rf95.setTxPower(23, false);

  /* INITIALIZING RTC */
  rtc.begin(); // To use the RTC, first begin it
  rtc.setTime(hours, minutes, seconds); // Then set the time
  rtc.setDate(day, month, year); // And the date
  Serial.println("RTC Started!");
  Serial.println("________________________________________");

  // Syntax for the LibreOffice macro
  Serial.println("CLEARDATA");
  Serial.println("LABEL,Num,Date,Time,Temp,Humid,sensX,sensY,sensZ");
}


/*------------------------ RECEIVER CODE ------------------------*/
void loop()
{
  if (rf95.available())
  {
    // Should be a message for us now   
    uint8_t buf[RH_RF95_MAX_MESSAGE_LEN];
    uint8_t len = sizeof(buf);
    
    if (rf95.recv(buf, &len))
    {
      digitalWrite(LED, HIGH);
      n++ ;
    
      delay(10);
      
      RH_RF95::printBuffer("Received: ", buf, len);

      // Union to convert Data
      union Temperature
      {
        float Temp;
        uint8_t pT[4];
      } T;
      
      union Humidity
      {
        float Humd;
        uint8_t pH[4];
      } H;

      union AccelX
      {
        float posX;
        uint8_t pX[4];
      } X;
      union AccelY
      {
        float posY;
        uint8_t pY[4];
      } Y;
      union AccelZ
      {
        float posZ;
        uint8_t pZ[4];
      } Z;
      
      memcpy(T.pT,buf,4);     // Copy 4 first bytes for Temp
      memcpy(H.pH,buf+4, 4);  // Copy 4 next bytes for Humid
      memcpy(X.pX,buf+8, 4);  // Copy 4 next bytes for X
      memcpy(Y.pY,buf+12, 4);  // Copy 4 next bytes for Y
      memcpy(Z.pZ,buf+16, 4);  // Copy 4 next bytes for Z
      
      /* To copy Data in the LibreOffice macro */
      Serial.print("DATA,");
      Serial.print(n);              // Data Receipt Number
      Serial.print(",DATE,TIME,");  // Date and Time
      Serial.print(T.Temp);         // Temperature
      Serial.print(",");
      Serial.print(H.Humd);         // Humidity
      Serial.print(",");
      Serial.print(X.posX);         // Sensitivity X
      Serial.print(",");
      Serial.print(Y.posY);         // Sensitivity Y
      Serial.print(",");
      Serial.println(Z.posZ);       // Sensitivity Z
      
      Serial.print("RSSI: ");
      Serial.println(rf95.lastRssi(), DEC);

      
      // Send a reply
      uint8_t data[] = "OK";
      rf95.send(data, sizeof(data));
      rf95.waitPacketSent();
      Serial.println("Sent a reply");
      Serial.println("________________________________________");
      digitalWrite(LED, LOW);
    }
    else
    {
      Serial.println("Receive failed");
    }
  }
}


/*_________________________________OTHER TOOLS_________________________________

// PRINT DATE FOR SERIAL IDE ARDUINO //
      Serial.print("Data received the ");
       // Print date...
      Serial.print(rtc.getDay());
      Serial.print("/");
      Serial.print(rtc.getMonth());
      Serial.print("/");
      Serial.print(rtc.getYear());
      Serial.print(" at ");
  
      // ...and time
      Serial.print(rtc.getHours());
      Serial.print(":");
      Serial.print(rtc.getMinutes());
      Serial.print(":");
      Serial.println(rtc.getSeconds());

// PRINT DATA FOR SERIAL IDE ARDUINO //
      Serial.println("Got: ");
      Serial.print("Temperature (°C) = ");
      Serial.print(T.Temp);
      Serial.print("Humidity (%) = ");
      Serial.print(H.Humd);
      Serial.println("Sensitivity (mg/LSB) :");
      Serial.print("X = ");
      Serial.print(X.posX);
      Serial.print("Y = ");
      Serial.print(Y.posY);
      Serial.print("Z = ");
      Serial.println(Z.posZ);


 */


