
#include <Wire.h>

#include "FXLS8471Q-P.h"



FXLS8471Q::FXLS8471Q()
{
  //Set initial values for private vars
}



//Start I2C communication
void FXLS8471Q::begin(TwoWire &wirePort)
{
    _i2cPort = &wirePort; //Grab which port the user wants us to use
    _i2cPort->begin();

    // write 0000 0000 = 0x00 to accelerometer control register 1 to place FXLS8471Q into
    // standby
    // [7-1] = 0000 000
    // [0]: active=0
    _i2cPort->beginTransmission(FXLS8471Q_SLAVE_ADDR);
    _i2cPort->write(FXLS8471Q_CTRL_REG1);
    _i2cPort->write(0x00);
    _i2cPort->endTransmission();

    // write 0000 0001= 0x01 to XYZ_DATA_CFG register
    // [7]: reserved
    // [6]: reserved
    // [5]: reserved
    // [4]: hpf_out=0
    // [3]: reserved
    // [2]: reserved
    // [1-0]: fs=00 for accelerometer range of +/-2g with 0.244mg/LSB
    _i2cPort->beginTransmission(FXLS8471Q_SLAVE_ADDR);
    _i2cPort->write(FXLS8471Q_XYZ_DATA_CFG);
    _i2cPort->write(0x00);
    _i2cPort->endTransmission();

    // write 0001 0101b = 0x15 to accelerometer control register 1
    // [7-6]: aslp_rate=00
    // [5-3]: dr=010 for 200Hz data rate
    // [2]: lnoise=1 for low noise mode
    // [1]: f_read=0 for normal 16 bit reads
    // [0]: active=1 to take the part out of standby and enable sampling
    _i2cPort->beginTransmission(FXLS8471Q_SLAVE_ADDR);
    _i2cPort->write(FXLS8471Q_CTRL_REG1);
    _i2cPort->write(0x15);
    _i2cPort->endTransmission();
}


#define MAX_WAIT 100
#define DELAY_INTERVAL 10
#define MAX_COUNTER (MAX_WAIT/DELAY_INTERVAL)

uint16_t FXLS8471Q::readValue(byte cmd)
{
  //Request a reading
  _i2cPort->beginTransmission(FXLS8471Q_SLAVE_ADDR);
  _i2cPort->write(cmd); //Measure value (prefer no hold!)
  _i2cPort->endTransmission();

  //Hang out while measurement is taken. datasheet says 50ms, practice may call for more
  bool validResult;
  byte counter;
  for (counter = 0, validResult = 0 ; counter < MAX_COUNTER && !validResult ; counter++)
  {
    delay(DELAY_INTERVAL);

    //Comes back in three bytes, data(MSB) / data(LSB) / Checksum
    validResult = (3 == _i2cPort->requestFrom(FXLS8471Q_SLAVE_ADDR, 3));
  }

  if (!validResult) return (ERROR_I2C_TIMEOUT); //Error out

  byte msb, lsb, checksum;
  msb = _i2cPort->read();
  lsb = _i2cPort->read();
  checksum = _i2cPort->read();

  uint16_t rawValue = ((uint16_t) msb << 8) | (uint16_t) lsb;

  if (checkCRC(rawValue, checksum) != 0) return (ERROR_BAD_CRC); //Error out

  return rawValue & 0xFFFC; // Zero out the status bits
}



//--------------------------- READ CODES ---------------------------

void FXLS8471Q::ReadAccel(float * x, float * y, float * z)
{
    int16_t ix, iy, iz ;
    int i;
    uint8_t Buffer[FXLS8471Q_READ_LEN]; // read buffer

    // read FXLS8471Q_READ_LEN=7 bytes (status byte and the three channels of data)
    _i2cPort->beginTransmission(FXLS8471Q_SLAVE_ADDR);
    _i2cPort->write(FXLS8471Q_STATUS);
    _i2cPort->endTransmission();
    _i2cPort->requestFrom(FXLS8471Q_SLAVE_ADDR,7);

    for(i=0;i<FXLS8471Q_READ_LEN;i++){
        Buffer[i] = _i2cPort->read();
    }

    ix = ((Buffer[1] * 256) + Buffer[2]);// / 4;
    iy = ((Buffer[3] * 256) + Buffer[4]);// / 4;
    iz = ((Buffer[5] * 256) + Buffer[6]);// / 4;
    // copy the 14 bit accelerometer byte data into 16 bit words
    *x = ((float) ix) / 16384.0;
    *y = ((float) iy) / 16384.0;
    *z = ((float) iz) / 16384.0;
}


uint8_t FXLS8471Q::getWhoAmI(void)
{
    uint8_t ID ;
    _i2cPort->beginTransmission(FXLS8471Q_SLAVE_ADDR);
    _i2cPort->write(FXLS8471Q_WHOAMI);
    _i2cPort->endTransmission();
    _i2cPort->requestFrom(FXLS8471Q_SLAVE_ADDR,1);
    ID = _i2cPort->read();
    return(ID);
}
